#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

RO = 1
WO = 2
RW = 3

class DefaultType:
    Date        = "Date"
    Time        = "Time"
    List        = "List"
    Dict        = "Dict"
    ResultSet   = "ResultSet"
    Integer     = "Integer"
    Long        = "Long"
    ULong       = "ULong"
    Unicode     = "Unicode"
    Blob        = "Blob"
    Boolean     = "Boolean"
    IP          = "IP"
    Double      = "Double"
    ListInfo    = "ListInfo"
    Ordering    = "Ordering"

default_types = [DefaultType.Date, DefaultType.Time, DefaultType.List,\
                DefaultType.Dict, DefaultType.ResultSet, DefaultType.Blob,
                DefaultType.IP, DefaultType.Long, DefaultType.ULong,
                DefaultType.Double, DefaultType.ListInfo, DefaultType.Ordering]

prefix = u""


def set_prefix(value):
    global prefix
    prefix = value


def get_prefix():
    return prefix


def get_access(access):
    if "RW" == access:
        return RW
    elif "RO" == access:
        return RO
    elif "WO" == access:
        return WO
    raise ValueError("Incorrect access type %s" % access)


class obj_with_param(object):
    def __init__(self, sub_type):
        self.sub_type = sub_type
        self.name = sub_type
        self.is_optional = False
        self.default_value = None

    def real_name(self):
        return u"%s%s" % (prefix, self.name)


class list_param(obj_with_param):
    name = DefaultType.List
    pass


class dict_param(obj_with_param):
    name = DefaultType.Dict
    pass


class result_set_param(obj_with_param):
    name = DefaultType.ResultSet
    pass


def optional_param(sub_type):
    real_sub_type, default_value = sub_type.split(",")
    temp = extract_type(real_sub_type)
    temp.default_value = default_value.strip()
    if temp.default_value == "void":
        temp.default_value = None
    temp.is_optional = True
    return temp


class gen_obj(object):
    def __init__(self, real_type):
        self.name = real_type
        self.is_optional = False
        self.default_value = None

    @property
    def real_name(self):
        return u"%s%s" % (prefix, self.name)


def extract_sub_type(param_type, in_sep="<", out_sep=">"):
    assert(in_sep in param_type and param_type.endswith(out_sep)), "Wrong type\
    for %s" % param_type
    sub_type = param_type[param_type.find(in_sep)+1:-1]
    return sub_type


def extract_type(param_type):
    if param_type.startswith("List<"):
        return list_param(extract_sub_type(param_type))
    elif param_type.startswith("Dict<"):
        return dict_param(extract_sub_type(param_type))
    elif param_type.startswith("ResultSet<"):
        return result_set_param(extract_sub_type(param_type))
    elif param_type.startswith("Optional<"):
        return optional_param(extract_sub_type(param_type))
    else:
        return gen_obj(param_type)


def add_to_import_list(imports, obj):
    if isinstance(obj, list_param):
        if DefaultType.List not in imports:
            imports.append(DefaultType.List)
    elif isinstance(obj, dict_param):
        if DefaultType.Dict not in imports:
            imports.append(DefaultType.Dict)
    elif isinstance(obj, result_set_param):
        if DefaultType.ResultSet not in imports:
            imports.append(DefaultType.ResultSet)

    if obj.name in imports:
        pass
    else:
        imports.append(obj.name)


class Attribute(object):

    def __init__(self, name, valid_type, access, hidden, validator):
        self.name       = name
        self.type       = valid_type
        self._access    = access
        self.hidden     = hidden
        self.comments   = None
        self.validator  = validator

    def read(self):
        return self._access in [RW, RO]

    def write(self):
        return self._access in [RW, WO]

    def __str__(self):
        return "(%s %s)" % (self.type, self.name)


class Object(object):
    def __init__(self, name, namespaces, import_path, parent_name):
        self.name           = name
        self.namespaces     = namespaces
        self.namespaces['default'] = self.namespaces['python']
        self.attributes     = {}
        self.imports        = []
        self.validator      = None

        self.backend_path   = import_path
        self.version        = None
        self.parent_name    = parent_name
        self.parent         = None

    def real_name(self, object_type, separator):

        return u"%s%s%s%s%s" % (prefix, separator, separator.join(self.namespaces[object_type].split(".")), separator, self.name)

    def add_attribute(self, name, valid_type, access, hidden=False,\
            validator=None):
        self.attributes[name] = Attribute(name, valid_type, access, hidden,\
                validator)
        add_to_import_list(self.imports, valid_type)

    def get_import_path(self, version, namespace):
        try:
            if self.version.is_lower(version):
                v = version
            else:
                v = self.version
            return "v%s.sv%s.%s" % (v.version, v.sub_version, self.namespaces[namespace])
        except KeyError:
            return u"gen"

    def get_backend_path(self):
        return self.backend_path

    def set_parent(self, parent):
        self.parent = parent

    def get_attributes(self):
        attribute_dict = {}
        if self.parent:
            for k, v in self.parent.get_attributes().iteritems():
                attribute_dict[k] = v
        for k, v in self.attributes.iteritems():
            attribute_dict[k] = v

        return attribute_dict

    def get_imports(self):
        imports = self.imports
        if self.parent:
            for imp in self.parent.get_imports():
                if imp not in imports:
                    imports.append(imp)
        return imports



    def __str__(self):
        string = "#### %s ####\n" % self.name
        return string + str(self.attributes)


class Service(object):
    def __init__(self, name, namespaces, identification, implementation, interface=False):
        self.name           = name
        self.functions      = {}
        self.objects_type   = []
        self.namespaces     = namespaces
        self.namespaces['default'] = self.namespaces['python']
        self.implementation = implementation
        self.identification = identification
        self.imports        = []
        self.comments       = ""
        self.extends        = []
        self.interface      = interface

    def add_function(self, authentication_list, func):
        func.authentication = authentication_list
        assert(not (func.func_name in self.functions)), "Function %s\
 already defined" % func.func_name
        self.functions[func.func_name] = func

    def get_import_path(self, version, namespace, create_folder=False):
        try:
            if self.version.is_lower(version):
                v = version
            else:
                v = self.version
            return "v%s.sv%s.%s" % (v.version, v.sub_version, self.namespaces[namespace])

        except KeyError:
            return u"gen"

    def add_import_types(self, function):
        for i in range(0, 4):
            for p, t in function.params[i]:
                add_to_import_list(self.imports, t)

    def get_imports(self):
        return self.imports

    def __str__(self):
        string = "@@@@ %s @@@@\n" % self.name
        for element in self.functions:
            return_type, params = self.functions[element]
            string += "%s %s %s\n" % (return_type, element, params)
        return string


class Enum(object):
    def __init__(self, name, sub_type, namespaces):
        self.name       = name
        self.sub_type   = sub_type
        self.values     = {}
        self.namespaces = namespaces
        self.namespaces['default'] = self.namespaces['python']

    @property
    def real_name(self):
        return u"%s%s" % (prefix, self.name)

    def get_import_path(self, version, namespace, create_folder=False):
        try:
            if self.version.is_lower(version):
                v = version
            else:
                v = self.version
            return "v%s.sv%s.%s" % (v.version, v.sub_version, self.namespaces[namespace])
        except KeyError:
            return u"gen"

    def add_value(self, key, value):
        self.values[key] = check_value(self.sub_type, value)

d_type = {
            "Unicode": unicode,
            "Integer": int,
        }


def check_value(type_string, value):
    assert (type_string in d_type), "Unknown type %s" % type_string
    return d_type[type_string](value)
